@@ -124,7 +124,7 @@ ephy_file_download_dir (void)
  * Returns a proper downloads destination by checking the
  * EPHY_PREFS_STATE_DOWNLOAD_DIR GSettings key and following this logic:
  *
- *  - Under flatpak, always use the XDG downloads directory
+ *  - Under sandbox, always use the XDG downloads directory
  *
  *  - An absolute path: considered user-set, use this value directly.
  *
@@ -142,7 +142,7 @@ ephy_file_get_downloads_dir (void)
   g_autofree char *download_dir = g_settings_get_string (EPHY_SETTINGS_STATE,
                                                          EPHY_PREFS_STATE_DOWNLOAD_DIR);
 
-  if (ephy_is_running_inside_flatpak ())
+  if (ephy_is_running_inside_sandbox ())
     return ephy_file_download_dir ();
 
   if (g_strcmp0 (download_dir, "Desktop") == 0)
@@ -376,7 +376,7 @@ ephy_file_helpers_init (const char            *profile_dir,
 
     app_file = g_build_filename (profile_dir, ".app", NULL);
     if (g_file_test (app_file, G_FILE_TEST_EXISTS)) {
-      const char *app_name = ephy_web_application_get_program_name_from_profile_directory (profile_dir_global);
+      const char *app_name = ephy_web_application_get_gapplication_id_from_profile_directory (profile_dir_global);
       cache_dir = g_build_filename (g_get_user_cache_dir (), app_name, NULL);
       config_dir = g_build_filename (g_get_user_config_dir (), app_name, NULL);
       profile_dir_type = EPHY_PROFILE_DIR_WEB_APP;
@@ -551,10 +551,10 @@ launch_application (GAppInfo *app,
   GdkScreen *screen;
   gboolean res;
 
-  /* This is impossible to implement inside flatpak. Higher layers must
+  /* This is impossible to implement inside sandbox. Higher layers must
    * ensure we don't get here.
    */
-  g_assert (!ephy_is_running_inside_flatpak ());
+  g_assert (!ephy_is_running_inside_sandbox ());
 
   display = gdk_display_get_default ();
   screen = gdk_screen_get_default ();
@@ -587,11 +587,11 @@ ephy_file_launch_desktop_file (const char                   *filename,
 {
   g_autoptr (GDesktopAppInfo) app = NULL;
 
-  /* This is impossible to implement inside flatpak. Higher layers must
+  /* This is impossible to implement inside sandbox. Higher layers must
    * ensure we don't get here.
    */
   g_assert (tag == EPHY_FILE_HELPERS_I_UNDERSTAND_I_MUST_NOT_USE_THIS_FUNCTION_UNDER_FLATPAK);
-  g_assert (!ephy_is_running_inside_flatpak ());
+  g_assert (!ephy_is_running_inside_sandbox ());
 
   app = g_desktop_app_info_new (filename);
 
@@ -641,11 +641,11 @@ ephy_file_launch_handler (GFile   *file,
 
   g_assert (file != NULL);
 
-  /* Launch via URI handler only under flatpak, because this way loses
+  /* Launch via URI handler only under sandbox, because this way loses
    * focus stealing prevention. There's no other way to open a file
-   * under flatpak, and focus stealing prevention becomes the
+   * under sandbox, and focus stealing prevention becomes the
    * responsibility of the portal in this case anyway. */
-  if (ephy_is_running_inside_flatpak ())
+  if (ephy_is_running_inside_sandbox ())
     return launch_via_uri_handler (file);
 
   app = g_file_query_default_handler (file, NULL, &error);
@@ -673,11 +673,11 @@ open_in_default_handler (const char                   *uri,
   g_autoptr (GError) error = NULL;
   GList uris;
 
-  /* This is impossible to implement inside flatpak. Higher layers must
+  /* This is impossible to implement inside sandbox. Higher layers must
    * ensure we don't get here.
    */
   g_assert (tag == EPHY_FILE_HELPERS_I_UNDERSTAND_I_MUST_NOT_USE_THIS_FUNCTION_UNDER_FLATPAK);
-  g_assert (!ephy_is_running_inside_flatpak ());
+  g_assert (!ephy_is_running_inside_sandbox ());
 
   context = gdk_display_get_app_launch_context (screen ? gdk_screen_get_display (screen) : gdk_display_get_default ());
   gdk_app_launch_context_set_screen (context, screen);
@@ -726,7 +726,7 @@ ephy_file_browse_to (GFile   *file,
 {
   g_autofree char *uri = g_file_get_uri (file);
 
-  if (ephy_is_running_inside_flatpak ()) {
+  if (ephy_is_running_inside_sandbox ()) {
     ephy_open_directory_via_flatpak_portal (uri);
     return TRUE;
   }
@@ -854,3 +854,55 @@ ephy_open_incognito_window (const char *uri)
 
   g_free (command);
 }
+
+void
+ephy_copy_directory (const char *source,
+                     const char *target)
+{
+  g_autoptr (GError) error = NULL;
+  GFileType type;
+  g_autoptr (GFile) src_file = g_file_new_for_path (source);
+  g_autoptr (GFile) dest_file = g_file_new_for_path (target);
+
+  type = g_file_query_file_type (src_file, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL);
+
+  if (type == G_FILE_TYPE_DIRECTORY) {
+    g_autoptr (GFileEnumerator) enumerator = NULL;
+
+    if (!g_file_make_directory_with_parents (dest_file, NULL, &error)) {
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
+        g_warning ("Could not create target directory for webextension: %s", error->message);
+        return;
+      }
+
+      g_error_free (error);
+    }
+
+    if (!g_file_copy_attributes (src_file, dest_file, G_FILE_COPY_NONE, NULL, &error)) {
+      g_warning ("Could not copy file attributes for webextension: %s", error->message);
+      return;
+    }
+
+    enumerator = g_file_enumerate_children (src_file, G_FILE_ATTRIBUTE_STANDARD_NAME, G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, &error);
+    if (!enumerator) {
+      g_warning ("Could not create file enumberator for webextensions: %s", error->message);
+      return;
+    }
+
+    for (;;) {
+      g_autoptr (GFileInfo) info = g_file_enumerator_next_file (enumerator, NULL, NULL);
+      if (!info)
+        break;
+      ephy_copy_directory (
+        g_build_filename (source, g_file_info_get_name (info), NULL),
+        g_build_filename (target, g_file_info_get_name (info), NULL));
+    }
+  } else if (type == G_FILE_TYPE_REGULAR) {
+    if (!g_file_copy (src_file, dest_file, G_FILE_COPY_NONE, NULL, NULL, NULL, &error)) {
+      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
+        g_warning ("Could not copy file for webextensions: %s", error->message);
+        return;
+      }
+    }
+  }
+}
