@@ -27,10 +27,13 @@ G_BEGIN_DECLS
 
 #define EPHY_TYPE_NOTIFICATION (ephy_notification_get_type ())
 
-G_DECLARE_FINAL_TYPE (EphyNotification, ephy_notification, EPHY, NOTIFICATION, GtkFrame)
+G_DECLARE_FINAL_TYPE (EphyNotification, ephy_notification, EPHY, NOTIFICATION, GtkBin)
 
 EphyNotification *ephy_notification_new  (const char *head,
                                           const char *body);
 void              ephy_notification_show (EphyNotification *self);
 
+gboolean          ephy_notification_is_duplicate (EphyNotification *notification_a,
+                                                  EphyNotification *notification_b);
+
 G_END_DECLS
