@@ -655,6 +655,8 @@ take_fresh_snapshot_in_background_if_stale (EphySnapshotService *service,
                           data,
                           (GDestroyNotify)snapshot_async_data_free);
     ephy_snapshot_service_take_from_webview (task);
+  } else {
+    snapshot_async_data_free (data);
   }
 }
 
@@ -673,7 +675,6 @@ got_snapshot_path_for_url (EphySnapshotService *service,
                            GAsyncResult        *result,
                            GTask               *task)
 {
-#ifndef __clang_analyzer__
   SnapshotAsyncData *data = g_task_get_task_data (task);
   char *path;
 
@@ -685,7 +686,6 @@ got_snapshot_path_for_url (EphySnapshotService *service,
   } else {
     ephy_snapshot_service_take_from_webview (task);
   }
-#endif
 }
 
 void
