@@ -25,19 +25,16 @@
 
 #include <gtk/gtk.h>
 
+#include "ephy-adaptive-mode.h"
+#include "ephy-bookmark-states.h"
 #include "ephy-security-levels.h"
 
 G_BEGIN_DECLS
 
 #define EPHY_TYPE_LOCATION_ENTRY (ephy_location_entry_get_type())
 
-G_DECLARE_FINAL_TYPE (EphyLocationEntry, ephy_location_entry, EPHY, LOCATION_ENTRY, GtkOverlay)
+G_DECLARE_FINAL_TYPE (EphyLocationEntry, ephy_location_entry, EPHY, LOCATION_ENTRY, GtkBin)
 
-typedef enum {
-  EPHY_LOCATION_ENTRY_BOOKMARK_ICON_HIDDEN,
-  EPHY_LOCATION_ENTRY_BOOKMARK_ICON_EMPTY,
-  EPHY_LOCATION_ENTRY_BOOKMARK_ICON_BOOKMARKED
-} EphyLocationEntryBookmarkIconState;
 
 GtkWidget      *ephy_location_entry_new                        (void);
 
@@ -51,8 +48,8 @@ void            ephy_location_entry_undo_reset                 (EphyLocationEntr
 
 void            ephy_location_entry_focus                      (EphyLocationEntry *entry);
 
-void            ephy_location_entry_set_bookmark_icon_state    (EphyLocationEntry                  *entry,
-                                                                EphyLocationEntryBookmarkIconState  state);
+void            ephy_location_entry_set_bookmark_icon_state    (EphyLocationEntry     *entry,
+                                                                EphyBookmarkIconState  state);
 
 void            ephy_location_entry_set_lock_tooltip           (EphyLocationEntry *entry,
                                                                 const char        *tooltip);
@@ -84,7 +81,7 @@ void            ephy_location_entry_page_action_add            (EphyLocationEntr
 
 void            ephy_location_entry_page_action_clear          (EphyLocationEntry *entry);
 
-void            ephy_location_entry_set_mobile_popdown         (EphyLocationEntry *entry,
-                                                                gboolean           mobile_popdown);
+void            ephy_location_entry_set_adaptive_mode          (EphyLocationEntry *entry,
+                                                                EphyAdaptiveMode   adpative_mode);
 
 G_END_DECLS
