@@ -22,7 +22,6 @@
 #include "ephy-security-popover.h"
 
 #include <glib/gi18n.h>
-#include <libsoup/soup.h>
 
 #include "ephy-certificate-dialog.h"
 #include "ephy-embed-shell.h"
@@ -131,19 +130,17 @@ ephy_security_popover_set_address (EphySecurityPopover *popover,
                                    const char          *address)
 {
   EphyPermissionsManager *permissions_manager;
-  SoupURI *uri;
+  g_autoptr (GUri) uri = NULL;
   g_autofree gchar *origin = NULL;
   g_autofree gchar *uri_text = NULL;
 
-  uri = soup_uri_new (address);
-  uri_text = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>", uri->host);
+  uri = g_uri_parse (address, G_URI_FLAGS_NONE, NULL);
+  uri_text = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>", g_uri_get_host (uri));
   /* Label when clicking the lock icon on a secure page. %s is the website's hostname. */
   gtk_label_set_markup (GTK_LABEL (popover->host_label), uri_text);
 
   popover->address = g_strdup (address);
-  popover->hostname = g_strdup (uri->host);
-
-  soup_uri_free (uri);
+  popover->hostname = g_strdup (g_uri_get_host (uri));
 
   origin = ephy_uri_to_security_origin (address);
   if (!origin)
@@ -250,7 +247,6 @@ ephy_security_popover_constructed (GObject *object)
 
   certificate_button = gtk_button_new_with_mnemonic (_("_View Certificate…"));
   gtk_widget_set_halign (certificate_button, GTK_ALIGN_END);
-  gtk_widget_set_valign (certificate_button, GTK_ALIGN_END);
   gtk_widget_set_margin_top (certificate_button, 5);
   gtk_widget_set_receives_default (certificate_button, FALSE);
   gtk_widget_show (certificate_button);
@@ -503,12 +499,13 @@ add_permission_combobox (EphySecurityPopover *popover,
   GtkWidget *hbox;
   GtkWidget *tmp;
 
-  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
   gtk_grid_attach (GTK_GRID (popover->grid), hbox, 0, popover->permission_pos++, 2, 1);
 
   tmp = gtk_label_new (name);
   gtk_label_set_xalign (GTK_LABEL (tmp), 0.0);
-  gtk_box_pack_start (GTK_BOX (hbox), tmp, TRUE, TRUE, 6);
+  gtk_widget_set_hexpand (tmp, TRUE);
+  gtk_box_pack_start (GTK_BOX (hbox), tmp, FALSE, TRUE, 0);
 
   widget = gtk_combo_box_text_new ();
   gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (widget), _("Allow"));
@@ -519,7 +516,7 @@ add_permission_combobox (EphySecurityPopover *popover,
     gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (widget), _(name));
   }
 
-  gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, FALSE, 6);
+  gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, TRUE, 0);
   g_signal_connect (widget, "changed", G_CALLBACK (callback), popover);
   gtk_size_group_add_widget (size_group, widget);
 
@@ -537,20 +534,22 @@ ephy_security_popover_init (EphySecurityPopover *popover)
   popover->grid = gtk_grid_new ();
   gtk_grid_set_column_spacing (GTK_GRID (popover->grid), 12);
   gtk_grid_set_row_spacing (GTK_GRID (popover->grid), 6);
-  g_object_set (popover->grid, "margin", 10, NULL);
+  gtk_widget_set_margin_top (popover->grid, 10);
+  gtk_widget_set_margin_bottom (popover->grid, 10);
+  gtk_widget_set_margin_start (popover->grid, 10);
+  gtk_widget_set_margin_end (popover->grid, 10);
 
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
   gtk_widget_set_halign (box, GTK_ALIGN_CENTER);
-  gtk_widget_set_hexpand (box, TRUE);
 
   popover->lock_image = gtk_image_new ();
-  gtk_box_pack_start (GTK_BOX (box), popover->lock_image, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (box), popover->lock_image, FALSE, TRUE, 0);
 
   popover->host_label = gtk_label_new (NULL);
   gtk_label_set_line_wrap (GTK_LABEL (popover->host_label), TRUE);
   gtk_label_set_line_wrap_mode (GTK_LABEL (popover->host_label), PANGO_WRAP_WORD_CHAR);
   gtk_label_set_xalign (GTK_LABEL (popover->host_label), 0.0);
-  gtk_box_pack_start (GTK_BOX (box), popover->host_label, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (box), popover->host_label, FALSE, TRUE, 0);
 
   popover->security_label = gtk_label_new (NULL);
   gtk_label_set_line_wrap (GTK_LABEL (popover->security_label), TRUE);
@@ -576,17 +575,18 @@ ephy_security_popover_init (EphySecurityPopover *popover)
   popover->access_location_combobox = add_permission_combobox (popover, _("Location access"), on_access_location_combobox_changed, combo_box_size_group, FALSE, NULL);
   popover->access_microphone_combobox = add_permission_combobox (popover, _("Microphone access"), on_access_microphone_combobox_changed, combo_box_size_group, FALSE, NULL);
   popover->access_webcam_combobox = add_permission_combobox (popover, _("Webcam access"), on_access_webcam_combobox_changed, combo_box_size_group, FALSE, NULL);
-  popover->autoplay_combobox = add_permission_combobox (popover, _("Media autoplay"), on_autoplay_policy_combobox_changed, combo_box_size_group, FALSE, "Without Sound");
+  popover->autoplay_combobox = add_permission_combobox (popover, _("Media autoplay"), on_autoplay_policy_combobox_changed, combo_box_size_group, FALSE, _("Without Sound"));
 
   gtk_container_add (GTK_CONTAINER (popover), popover->grid);
   gtk_widget_show_all (popover->grid);
 }
 
-GtkWidget *ephy_security_popover_new (GtkWidget           *relative_to,
-                                      const char          *address,
-                                      GTlsCertificate     *certificate,
-                                      GTlsCertificateFlags tls_errors,
-                                      EphySecurityLevel    security_level)
+GtkWidget *
+ephy_security_popover_new (GtkWidget            *relative_to,
+                           const char           *address,
+                           GTlsCertificate      *certificate,
+                           GTlsCertificateFlags  tls_errors,
+                           EphySecurityLevel     security_level)
 {
   g_assert (address != NULL);
 
