@@ -276,6 +276,8 @@ on_find_urls_cb (gpointer service,
   if (!success)
     return;
 
+  if (self->urls)
+    ephy_history_url_list_free (self->urls);
   self->urls = ephy_history_url_list_copy (result_data);
 
   clear_listbox (self->listbox);
@@ -312,7 +314,7 @@ remove_pending_sorter_source (EphyHistoryDialog *self,
   g_clear_handle_id (&self->sorter_source, g_source_remove);
 
   if (free_urls && self->urls) {
-    ephy_history_host_list_free (self->urls);
+    ephy_history_url_list_free (self->urls);
     self->urls = NULL;
   }
 }
@@ -455,7 +457,6 @@ create_row (EphyHistoryDialog *self,
   g_object_set_data (G_OBJECT (row), "check-button", check_button);
   gtk_widget_set_valign (check_button, GTK_ALIGN_CENTER);
   gtk_widget_set_tooltip_text (check_button, _("Remove the selected pages from history"));
-  gtk_button_set_relief (GTK_BUTTON (check_button), GTK_RELIEF_NONE);
   g_signal_connect (check_button, "toggled", G_CALLBACK (row_check_button_toggled), self);
 
   /* Copy URL button */
@@ -622,8 +623,8 @@ on_key_press_event (EphyHistoryDialog *self,
     return GDK_EVENT_STOP;
 
   if (key->keyval == GDK_KEY_Down || key->keyval == GDK_KEY_Page_Down) {
-    GList *childrens = gtk_container_get_children (GTK_CONTAINER (self->listbox));
-    GtkWidget *last = g_list_last (childrens)->data;
+    g_autoptr (GList) children = gtk_container_get_children (GTK_CONTAINER (self->listbox));
+    GtkWidget *last = g_list_last (children)->data;
     GtkWidget *focus = gtk_container_get_focus_child (GTK_CONTAINER (self->listbox));
 
     if (focus == last) {
