@@ -21,9 +21,9 @@
 
 #pragma once
 
-#include <gtk/gtk.h>
+#include <handy.h>
 #include "ephy-adaptive-mode.h"
-#include "ephy-notebook.h"
+#include "ephy-tab-view.h"
 
 G_BEGIN_DECLS
 
@@ -31,10 +31,12 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE (EphyPageRow, ephy_page_row, EPHY, PAGE_ROW, GtkListBoxRow)
 
-EphyPageRow *ephy_page_row_new (EphyNotebook *notebook,
-                                int           position);
+EphyPageRow *ephy_page_row_new (EphyTabView *view,
+                                HdyTabPage  *page);
 
 void ephy_page_row_set_adaptive_mode (EphyPageRow      *self,
                                       EphyAdaptiveMode  adaptive_mode);
 
+HdyTabPage *ephy_page_row_get_page (EphyPageRow *self);
+
 G_END_DECLS
