@@ -28,6 +28,7 @@
 #include "ephy-embed-container.h"
 #include "ephy-embed-utils.h"
 #include "ephy-file-helpers.h"
+#include "ephy-firefox-sync-dialog.h"
 #include "ephy-gui.h"
 #include "ephy-header-bar.h"
 #include "ephy-history-dialog.h"
@@ -60,8 +61,10 @@ struct _EphyShell {
   EphyBookmarksManager *bookmarks_manager;
   EphyHistoryManager *history_manager;
   EphyOpenTabsManager *open_tabs_manager;
+  EphyWebExtensionManager *web_extension_manager;
   GNetworkMonitor *network_monitor;
   GtkWidget *history_dialog;
+  GtkWidget *firefox_sync_dialog;
   GObject *prefs_dialog;
   EphyShellStartupContext *local_startup_context;
   EphyShellStartupContext *remote_startup_context;
@@ -241,6 +244,18 @@ show_history (GSimpleAction *action,
   window_cmd_show_history (NULL, NULL, EPHY_WINDOW (window));
 }
 
+static void
+show_firefox_sync (GSimpleAction *action,
+                   GVariant      *parameter,
+                   gpointer       user_data)
+{
+  GtkWindow *window;
+
+  window = gtk_application_get_active_window (GTK_APPLICATION (ephy_shell));
+
+  window_cmd_show_firefox_sync (NULL, NULL, EPHY_WINDOW (window));
+}
+
 static void
 show_preferences (GSimpleAction *action,
                   GVariant      *parameter,
@@ -349,6 +364,7 @@ static GActionEntry app_entries[] = {
   { "export-bookmarks", export_bookmarks, NULL, NULL, NULL },
   { "import-passwords", import_passwords, NULL, NULL, NULL },
   { "history", show_history, NULL, NULL, NULL },
+  { "firefox-sync-dialog", show_firefox_sync, NULL, NULL, NULL },
   { "preferences", show_preferences, NULL, NULL, NULL },
   { "shortcuts", show_shortcuts, NULL, NULL, NULL },
   { "help", show_help, NULL, NULL, NULL },
@@ -479,6 +495,18 @@ ephy_shell_startup (GApplication *application)
 
   hdy_init ();
 
+  hdy_style_manager_set_color_scheme (hdy_style_manager_get_default (),
+                                      HDY_COLOR_SCHEME_PREFER_LIGHT);
+
+  /* If we are under Pantheon set the icon-theme and cursor-theme accordingly. */
+  if (is_desktop_pantheon ()) {
+    GtkSettings *settings = gtk_settings_get_default ();
+    g_object_set (settings,
+                  "gtk-icon-theme-name", "elementary",
+                  "gtk-cursor-theme-name", "elementary",
+                  NULL);
+  }
+
   /* We're not remoting; start our services */
 
   mode = ephy_embed_shell_get_mode (embed_shell);
@@ -610,7 +638,7 @@ ephy_shell_activate (GApplication *application)
   EphyShell *shell = EPHY_SHELL (application);
   EphyEmbedShell *embed_shell = EPHY_EMBED_SHELL (shell);
 
-  if (!is_desktop_gnome ()) {
+  if (!is_desktop_gnome () && !is_desktop_pantheon ()) {
     g_signal_connect (ephy_shell_get_net_monitor (shell), "notify::connectivity", G_CALLBACK (connectivity_changed), shell);
     portal_check (shell);
   }
@@ -855,6 +883,7 @@ ephy_shell_dispose (GObject *object)
   g_clear_object (&shell->bookmarks_manager);
   g_clear_object (&shell->history_manager);
   g_clear_object (&shell->open_tabs_manager);
+  g_clear_object (&shell->web_extension_manager);
 
   g_hash_table_destroy (shell->notifications);
 
@@ -910,23 +939,22 @@ webkit_notification_clicked_cb (WebKitNotification *notification,
 
   for (guint win_idx = 0; win_idx < g_list_length (windows); win_idx++) {
     EphyWindow *window;
-    GtkWidget *notebook;
+    EphyTabView *tab_view;
     int n_pages;
 
     window = EPHY_WINDOW (g_list_nth_data (windows, win_idx));
-
-    notebook = ephy_window_get_notebook (window);
-    n_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
+    tab_view = ephy_window_get_tab_view (window);
+    n_pages = ephy_tab_view_get_n_pages (tab_view);
 
     for (int i = 0; i < n_pages; i++) {
       EphyEmbed *embed;
       WebKitWebView *webview;
 
-      embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i));
+      embed = EPHY_EMBED (ephy_tab_view_get_nth_page (tab_view, i));
       webview = WEBKIT_WEB_VIEW (ephy_embed_get_web_view (embed));
 
       if (webview == notification_webview) {
-        gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), i);
+        ephy_tab_view_select_page (tab_view, GTK_WIDGET (embed));
         gtk_window_present (GTK_WINDOW (window));
         return;
       }
@@ -1000,6 +1028,7 @@ ephy_shell_new_tab_full (EphyShell       *shell,
   EphyEmbed *embed = NULL;
   gboolean jump_to = FALSE;
   int position = -1;
+  EphyEmbed *parent = NULL;
 
   g_assert (EPHY_IS_SHELL (shell));
   g_assert (EPHY_IS_WINDOW (window));
@@ -1015,7 +1044,7 @@ ephy_shell_new_tab_full (EphyShell       *shell,
 
   if (flags & EPHY_NEW_TAB_APPEND_AFTER) {
     if (previous_embed)
-      position = ephy_window_get_position_for_new_embed (window, previous_embed);
+      parent = previous_embed;
     else
       g_warning ("Requested to append new tab after parent, but 'previous_embed' was NULL");
   }
@@ -1036,7 +1065,7 @@ ephy_shell_new_tab_full (EphyShell       *shell,
                                     "progress-bar-enabled", ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_APPLICATION,
                                     NULL));
   gtk_widget_show (GTK_WIDGET (embed));
-  ephy_embed_container_add_child (EPHY_EMBED_CONTAINER (window), embed, position, jump_to);
+  ephy_embed_container_add_child (EPHY_EMBED_CONTAINER (window), embed, parent, position, jump_to);
 
   if ((flags & EPHY_NEW_TAB_DONT_SHOW_WINDOW) == 0 &&
       ephy_embed_shell_get_mode (embed_shell) != EPHY_EMBED_SHELL_MODE_TEST) {
@@ -1216,6 +1245,25 @@ ephy_shell_get_history_dialog (EphyShell *shell)
   return shell->history_dialog;
 }
 
+/**
+ * ephy_shell_get_firefox_sync_dialog:
+ *
+ * Return value: (transfer none):
+ **/
+GtkWidget *
+ephy_shell_get_firefox_sync_dialog (EphyShell *shell)
+{
+  if (shell->firefox_sync_dialog == NULL) {
+    shell->firefox_sync_dialog = ephy_firefox_sync_dialog_new ();
+    g_signal_connect (shell->firefox_sync_dialog,
+                      "destroy",
+                      G_CALLBACK (gtk_widget_destroyed),
+                      &shell->firefox_sync_dialog);
+  }
+
+  return shell->firefox_sync_dialog;
+}
+
 /**
  * ephy_shell_get_prefs_dialog:
  *
@@ -1245,9 +1293,10 @@ _ephy_shell_create_instance (EphyEmbedShellMode mode)
 
   if (mode == EPHY_EMBED_SHELL_MODE_APPLICATION) {
     const char *profile_dir = ephy_profile_dir ();
-    const char *web_id = ephy_web_application_get_program_name_from_profile_directory (profile_dir);
 
-    id = web_id;
+    id = ephy_web_application_get_gapplication_id_from_profile_directory (profile_dir);
+    if (id == NULL)
+      g_error ("Cannot start web app instance: %s is not a valid profile directory", profile_dir);
   } else {
     id = APPLICATION_ID;
   }
@@ -1492,3 +1541,59 @@ ephy_shell_startup_finished (EphyShell *shell)
 {
   return shell->startup_finished;
 }
+
+EphyWebExtensionManager *
+ephy_shell_get_web_extension_manager (EphyShell *shell)
+{
+  g_assert (EPHY_IS_SHELL (shell));
+
+  if (shell->web_extension_manager == NULL)
+    shell->web_extension_manager = ephy_web_extension_manager_new ();
+
+  return shell->web_extension_manager;
+}
+
+
+/* Helper functions: better place for this? */
+EphyWebView *
+ephy_shell_get_web_view (EphyShell *shell,
+                         guint64    id)
+{
+  GList *windows;
+  GtkWindow *window;
+  EphyTabView *tab_view;
+
+  windows = gtk_application_get_windows (GTK_APPLICATION (shell));
+
+  for (GList *list = windows; list && list->data; list = list->next) {
+    window = GTK_WINDOW (list->data);
+    tab_view = ephy_window_get_tab_view (EPHY_WINDOW (window));
+
+    for (int i = 0; i < ephy_tab_view_get_n_pages (tab_view); i++) {
+      GtkWidget *page = ephy_tab_view_get_nth_page (tab_view, i);
+      EphyWebView *web_view = ephy_embed_get_web_view (EPHY_EMBED (page));
+
+      if (ephy_web_view_get_uid (web_view) == id)
+        return web_view;
+    }
+  }
+
+  return NULL;
+}
+
+EphyWebView *
+ephy_shell_get_active_web_view (EphyShell *shell)
+{
+  GtkWindow *window;
+  EphyTabView *tab_view;
+  GtkWidget *page;
+
+  window = gtk_application_get_active_window (GTK_APPLICATION (shell));
+  if (!window)
+    return NULL;
+
+  tab_view = ephy_window_get_tab_view (EPHY_WINDOW (window));
+  page = ephy_tab_view_get_selected_page (tab_view);
+
+  return ephy_embed_get_web_view (EPHY_EMBED (page));
+}
