@@ -20,8 +20,11 @@
 #include "ephy-suggestion-model.h"
 
 #include "ephy-embed-shell.h"
+#include "ephy-prefs.h"
 #include "ephy-search-engine-manager.h"
+#include "ephy-settings.h"
 #include "ephy-suggestion.h"
+#include "ephy-user-agent.h"
 #include "ephy-window.h"
 
 #include <dazzle.h>
@@ -37,8 +40,16 @@ struct _EphySuggestionModel {
   GSequence *items;
   GCancellable *icon_cancellable;
   guint num_custom_entries;
+  SoupSession *session;
 };
 
+#define QUERY_SCOPE_ALL         ' '
+#define QUERY_SCOPE_TABS        '%'
+#define QUERY_SCOPE_SUGGESTIONS '?'
+#define QUERY_SCOPE_BOOKMARKS   '*'
+#define QUERY_SCOPE_HISTORY     '^'
+#define MAX_QUERY_SCOPES        4
+
 enum {
   PROP_0,
   PROP_BOOKMARKS_MANAGER,
@@ -62,6 +73,7 @@ ephy_suggestion_model_finalize (GObject *object)
   g_clear_object (&self->history_service);
   g_clear_pointer (&self->urls, g_sequence_free);
   g_clear_pointer (&self->items, g_sequence_free);
+  g_clear_object (&self->session);
 
   g_cancellable_cancel (self->icon_cancellable);
   g_clear_object (&self->icon_cancellable);
@@ -139,6 +151,7 @@ static void
 ephy_suggestion_model_init (EphySuggestionModel *self)
 {
   self->items = g_sequence_new (g_object_unref);
+  self->session = soup_session_new_with_options ("user-agent", ephy_user_agent_get (), NULL);
 }
 
 static GType
@@ -283,8 +296,16 @@ static gboolean
 append_suggestion (EphySuggestionModel *self,
                    EphySuggestion      *suggestion)
 {
+  if (g_sequence_lookup (self->urls, (char *)ephy_suggestion_get_uri (suggestion), (GCompareDataFunc)g_strcmp0, NULL))
+    return FALSE;
+
   if (self->num_custom_entries < MAX_URL_ENTRIES) {
-    g_sequence_append (self->items, suggestion);
+    const char *uri = ephy_suggestion_get_uri (suggestion);
+
+    g_sequence_append (self->items, g_object_ref (suggestion));
+    g_sequence_append (self->urls, g_strdup (uri));
+
+    load_favicon (self, suggestion, uri);
     self->num_custom_entries++;
 
     return TRUE;
@@ -293,218 +314,55 @@ append_suggestion (EphySuggestionModel *self,
   return FALSE;
 }
 
-static guint
-add_bookmarks (EphySuggestionModel *self,
-               const char          *query)
-{
-  g_autoptr (GList) new_urls = NULL;
-  GSequence *bookmarks;
-  guint added = 0;
-
-  bookmarks = ephy_bookmarks_manager_get_bookmarks (self->bookmarks_manager);
-
-  for (GSequenceIter *iter = g_sequence_get_begin_iter (bookmarks);
-       !g_sequence_iter_is_end (iter);
-       iter = g_sequence_iter_next (iter)) {
-    EphyBookmark *bookmark;
-    GSequence *tags;
-    const char *url, *title;
-
-    bookmark = g_sequence_get (iter);
-
-    url = ephy_bookmark_get_url (bookmark);
-    if (g_sequence_lookup (self->urls, (gpointer)url, (GCompareDataFunc)g_strcmp0, NULL))
-      continue;
-
-    title = ephy_bookmark_get_title (bookmark);
-    if (strlen (title) == 0)
-      title = url;
-
-    tags = ephy_bookmark_get_tags (bookmark);
-    if (should_add_bookmark_to_model (self, query, title, url, tags)) {
-      EphySuggestion *suggestion;
-      g_autofree gchar *escaped_title = NULL;
-      g_autofree gchar *markup = NULL;
-
-      escaped_title = g_markup_escape_text (title, -1);
-      markup = dzl_fuzzy_highlight (escaped_title, query, FALSE);
-      suggestion = ephy_suggestion_new (markup, title, url);
-      load_favicon (self, suggestion, url);
-      ephy_suggestion_set_secondary_icon (suggestion, "starred-symbolic");
-
-      if (append_suggestion (self, suggestion)) {
-        new_urls = g_list_prepend (new_urls, g_strdup (url));
-        added++;
-      } else {
-        break;
-      }
-    }
-  }
-
-  for (GList *p = new_urls; p != NULL; p = p->next)
-    g_sequence_append (self->urls, g_steal_pointer (&p->data));
-  g_sequence_sort (self->urls, (GCompareDataFunc)g_strcmp0, NULL);
-
-  return added;
-}
-
-static guint
-add_history (EphySuggestionModel *self,
-             GList               *urls,
-             const char          *query)
-{
-  guint added = 0;
-
-  for (const GList *p = urls; p != NULL; p = p->next) {
-    EphyHistoryURL *url = (EphyHistoryURL *)p->data;
-    EphySuggestion *suggestion;
-    g_autofree gchar *escaped_title = NULL;
-    g_autofree gchar *markup = NULL;
-    const gchar *title = url->title;
-
-    if (g_sequence_lookup (self->urls, url->url, (GCompareDataFunc)g_strcmp0,
-                           NULL))
-      continue;
-
-    if (strlen (url->title) == 0)
-      title = url->url;
-
-    escaped_title = g_markup_escape_text (title, -1);
-
-    markup = dzl_fuzzy_highlight (escaped_title, query, FALSE);
-    suggestion = ephy_suggestion_new (markup, title, url->url);
-    load_favicon (self, suggestion, url->url);
-
-    if (append_suggestion (self, suggestion))
-      added++;
-    else
-      break;
-  }
-
-  return added;
-}
-
 static guint
 add_search_engines (EphySuggestionModel *self,
                     const char          *query)
 {
   EphyEmbedShell *shell;
   EphySearchEngineManager *manager;
-  char **engines;
-  guint added = 0;
+  guint added;
 
   shell = ephy_embed_shell_get_default ();
   manager = ephy_embed_shell_get_search_engine_manager (shell);
-  engines = ephy_search_engine_manager_get_names (manager);
 
-  for (guint i = 0; engines[i] != NULL; i++) {
+  for (added = 0; added < g_list_model_get_n_items (G_LIST_MODEL (manager)); added++) {
+    g_autoptr (EphySearchEngine) engine = g_list_model_get_item (G_LIST_MODEL (manager), added);
     EphySuggestion *suggestion;
+    const char *engine_name;
     g_autofree char *address = NULL;
     g_autofree char *escaped_title = NULL;
     g_autofree char *markup = NULL;
-    g_autoptr (SoupURI) uri = NULL;
+    g_autoptr (GUri) uri = NULL;
 
-    address = ephy_search_engine_manager_build_search_address (manager, engines[i], query);
-    escaped_title = g_markup_escape_text (engines[i], -1);
+    engine_name = ephy_search_engine_get_name (engine);
+    address = ephy_search_engine_build_search_address (engine, query);
+    escaped_title = g_markup_escape_text (engine_name, -1);
     markup = dzl_fuzzy_highlight (escaped_title, query, FALSE);
-    suggestion = ephy_suggestion_new (markup, engines[i], address);
+    suggestion = ephy_suggestion_new (markup, engine_name, address);
 
-    uri = soup_uri_new (address);
+    uri = g_uri_parse (address, G_URI_FLAGS_NONE, NULL);
     if (uri) {
       g_free (address);
-      address = g_strdup_printf ("%s://%s/", soup_uri_get_scheme (uri), soup_uri_get_host (uri));
+      address = g_strdup_printf ("%s://%s/", g_uri_get_scheme (uri), g_uri_get_host (uri));
     }
 
     load_favicon (self, suggestion, address);
 
     g_sequence_append (self->items, suggestion);
-    added++;
   }
 
-  g_strfreev (engines);
-
-  return added;
-}
-
-static guint
-add_tabs (EphySuggestionModel *self,
-          const char          *query)
-{
-  GApplication *application;
-  EphyEmbedShell *shell;
-  EphyWindow *window;
-  GtkWidget *notebook;
-  GList *windows;
-  gint n_pages;
-  gint current;
-  guint added = 0;
-
-  shell = ephy_embed_shell_get_default ();
-  application = G_APPLICATION (shell);
-  windows = gtk_application_get_windows (GTK_APPLICATION (application));
-
-  for (guint win_idx = 0; win_idx < g_list_length (windows); win_idx++) {
-    window = EPHY_WINDOW (g_list_nth_data (windows, win_idx));
-
-    notebook = ephy_window_get_notebook (window);
-    n_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
-    current = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
-
-    for (int i = 0; i < n_pages; i++) {
-      EphyEmbed *embed;
-      EphyWebView *webview;
-      EphySuggestion *suggestion;
-      g_autofree gchar *escaped_title = NULL;
-      g_autofree gchar *markup = NULL;
-      const gchar *display_address;
-      const gchar *url;
-      g_autofree gchar *address = NULL;
-      const gchar *title;
-      g_autofree gchar *title_casefold = NULL;
-      g_autofree gchar *display_address_casefold = NULL;
-      g_autofree gchar *query_casefold = NULL;
-
-      if (win_idx == 0 && i == current)
-        continue;
-
-      embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i));
-      webview = ephy_embed_get_web_view (embed);
-      display_address = ephy_web_view_get_display_address (webview);
-      url = ephy_web_view_get_address (webview);
-      address = g_strdup_printf ("ephy-tab://%d@%d", i, win_idx);
-      title = webkit_web_view_get_title (WEBKIT_WEB_VIEW (webview));
-
-      display_address_casefold = g_utf8_casefold (display_address, -1);
-      query_casefold = g_utf8_casefold (query, -1);
-      if (!title)
-        title = "";
-
-      title_casefold = g_utf8_casefold (title, -1);
-
-      if ((title_casefold && strstr (title_casefold, query_casefold)) || strstr (display_address_casefold, query_casefold)) {
-        char *escaped_address = g_markup_escape_text (display_address, -1);
-
-        escaped_title = g_markup_escape_text (title, -1);
-        markup = dzl_fuzzy_highlight (escaped_title, query, FALSE);
-        suggestion = ephy_suggestion_new_with_custom_subtitle (markup, title, escaped_address, address);
-        load_favicon (self, suggestion, display_address);
-        ephy_suggestion_set_secondary_icon (suggestion, "go-jump-symbolic");
-
-        g_sequence_append (self->urls, g_strdup (url));
-        g_sequence_append (self->items, suggestion);
-        added++;
-      }
-    }
-  }
-
-  g_sequence_sort (self->urls, (GCompareDataFunc)g_strcmp0, NULL);
-
   return added;
 }
 
 typedef struct {
   char *query;
+  char scope;
   gboolean include_search_engines;
+  GSequence *tabs;
+  GSequence *bookmarks;
+  GSequence *history;
+  GSequence *google_suggestions;
+  int active_sources;
 } QueryData;
 
 static QueryData *
@@ -514,8 +372,26 @@ query_data_new (const char *query,
   QueryData *data;
 
   data = g_malloc0 (sizeof (QueryData));
-  data->query = g_strdup (query);
   data->include_search_engines = include_search_engines;
+  data->tabs = g_sequence_new (g_object_unref);
+  data->bookmarks = g_sequence_new (g_object_unref);
+  data->history = g_sequence_new (g_object_unref);
+  data->google_suggestions = g_sequence_new (g_object_unref);
+
+  /* Split the search string. */
+  if (strlen (query) > 1 && query[1] == ' ' &&
+      (query[0] == QUERY_SCOPE_TABS ||
+       query[0] == QUERY_SCOPE_BOOKMARKS ||
+       query[0] == QUERY_SCOPE_HISTORY ||
+       query[0] == QUERY_SCOPE_SUGGESTIONS)) {
+    data->query = g_strdup (query + 2);
+    data->scope = query[0];
+    data->active_sources = 1;
+  } else {
+    data->query = g_strdup (query);
+    data->scope = QUERY_SCOPE_ALL;
+    data->active_sources = MAX_QUERY_SCOPES;
+  }
 
   return data;
 }
@@ -524,26 +400,27 @@ static void
 query_data_free (QueryData *data)
 {
   g_assert (data != NULL);
-  g_free (data->query);
+  g_clear_pointer (&data->tabs, g_sequence_free);
+  g_clear_pointer (&data->bookmarks, g_sequence_free);
+  g_clear_pointer (&data->history, g_sequence_free);
+  g_clear_pointer (&data->google_suggestions, g_sequence_free);
+  g_clear_pointer (&data->query, g_free);
   g_free (data);
 }
 
 static void
-query_completed_cb (EphyHistoryService *service,
-                    gboolean            success,
-                    gpointer            result_data,
-                    gpointer            user_data)
+query_collection_done (EphySuggestionModel *self,
+                       GTask               *task)
 {
-  GTask *task = user_data;
-  EphySuggestionModel *self;
   QueryData *data;
-  GList *urls;
   guint removed;
   guint added = 0;
 
   self = g_task_get_source_object (task);
   data = g_task_get_task_data (task);
-  urls = (GList *)result_data;
+
+  if (--data->active_sources)
+    return;
 
   g_cancellable_cancel (self->icon_cancellable);
   g_clear_object (&self->icon_cancellable);
@@ -559,11 +436,48 @@ query_completed_cb (EphyHistoryService *service,
   self->num_custom_entries = 0;
 
   if (strlen (data->query) > 0) {
-    added = add_tabs (self, data->query);
-    added += add_bookmarks (self, data->query);
-    added += add_history (self, urls, data->query);
+    /* Search results have the following order:
+     * - Open Tabs
+     * - Search Suggestions
+     * - Bookmarks
+     * - History
+     * - Search Engines
+     */
+    for (GSequenceIter *iter = g_sequence_get_begin_iter (data->tabs); !g_sequence_iter_is_end (iter); iter = g_sequence_iter_next (iter)) {
+      EphySuggestion *tmp = g_sequence_get (iter);
 
-    if (data->include_search_engines)
+      g_sequence_append (self->items, g_object_ref (tmp));
+      added++;
+    }
+
+    for (GSequenceIter *iter = g_sequence_get_begin_iter (data->google_suggestions); !g_sequence_iter_is_end (iter); iter = g_sequence_iter_next (iter)) {
+      EphySuggestion *tmp = g_sequence_get (iter);
+
+      if (append_suggestion (self, tmp))
+        added++;
+      else
+        break;
+    }
+
+    for (GSequenceIter *iter = g_sequence_get_begin_iter (data->bookmarks); !g_sequence_iter_is_end (iter); iter = g_sequence_iter_next (iter)) {
+      EphySuggestion *tmp = g_sequence_get (iter);
+
+      if (append_suggestion (self, tmp))
+        added++;
+      else
+        break;
+    }
+
+    for (GSequenceIter *iter = g_sequence_get_begin_iter (data->history); !g_sequence_iter_is_end (iter); iter = g_sequence_iter_next (iter)) {
+      EphySuggestion *tmp = g_sequence_get (iter);
+
+      if (append_suggestion (self, tmp))
+        added++;
+      else
+        break;
+    }
+
+    if (data->scope == QUERY_SCOPE_ALL && data->include_search_engines)
       added += add_search_engines (self, data->query);
   }
 
@@ -573,6 +487,260 @@ query_completed_cb (EphyHistoryService *service,
   g_object_unref (task);
 }
 
+static void
+tabs_query (EphySuggestionModel *self,
+            QueryData           *data,
+            GTask               *task)
+{
+  GApplication *application;
+  EphyEmbedShell *shell;
+  EphyWindow *window;
+  EphyTabView *tab_view;
+  GList *windows;
+  gint n_pages, selected;
+
+  shell = ephy_embed_shell_get_default ();
+  application = G_APPLICATION (shell);
+  windows = gtk_application_get_windows (GTK_APPLICATION (application));
+
+  for (guint win_idx = 0; win_idx < g_list_length (windows); win_idx++) {
+    window = EPHY_WINDOW (g_list_nth_data (windows, win_idx));
+
+    tab_view = ephy_window_get_tab_view (window);
+    n_pages = ephy_tab_view_get_n_pages (tab_view);
+    selected = ephy_tab_view_get_selected_index (tab_view);
+
+    for (int i = 0; i < n_pages; i++) {
+      EphyEmbed *embed;
+      EphyWebView *webview;
+      EphySuggestion *suggestion;
+      g_autofree gchar *escaped_title = NULL;
+      g_autofree gchar *markup = NULL;
+      const gchar *display_address;
+      g_autofree gchar *address = NULL;
+      const gchar *title;
+      g_autofree gchar *title_casefold = NULL;
+      g_autofree gchar *display_address_casefold = NULL;
+      g_autofree gchar *query_casefold = NULL;
+
+      if (win_idx == 0 && i == selected)
+        continue;
+
+      embed = EPHY_EMBED (ephy_tab_view_get_nth_page (tab_view, i));
+      webview = ephy_embed_get_web_view (embed);
+      display_address = ephy_web_view_get_display_address (webview);
+      address = g_strdup_printf ("ephy-tab://%d@%d", i, win_idx);
+      title = webkit_web_view_get_title (WEBKIT_WEB_VIEW (webview));
+
+      display_address_casefold = g_utf8_casefold (display_address, -1);
+      query_casefold = g_utf8_casefold (data->query, -1);
+      if (!title)
+        title = "";
+
+      title_casefold = g_utf8_casefold (title, -1);
+
+      if ((title_casefold && strstr (title_casefold, query_casefold)) || strstr (display_address_casefold, query_casefold)) {
+        char *escaped_address = g_markup_escape_text (display_address, -1);
+
+        escaped_title = g_markup_escape_text (title, -1);
+        markup = dzl_fuzzy_highlight (escaped_title, data->query, FALSE);
+        suggestion = ephy_suggestion_new_with_custom_subtitle (markup, title, escaped_address, address);
+        ephy_suggestion_set_secondary_icon (suggestion, "go-jump-symbolic");
+
+        g_sequence_append (data->tabs, g_object_ref (suggestion));
+      }
+    }
+  }
+
+  query_collection_done (self, g_steal_pointer (&task));
+}
+
+static void
+bookmarks_query (EphySuggestionModel *self,
+                 QueryData           *data,
+                 GTask               *task)
+{
+  g_autoptr (GList) new_urls = NULL;
+  GSequence *bookmarks;
+
+  bookmarks = ephy_bookmarks_manager_get_bookmarks (self->bookmarks_manager);
+
+  for (GSequenceIter *iter = g_sequence_get_begin_iter (bookmarks);
+       !g_sequence_iter_is_end (iter);
+       iter = g_sequence_iter_next (iter)) {
+    EphyBookmark *bookmark;
+    GSequence *tags;
+    const char *url, *title;
+
+    bookmark = g_sequence_get (iter);
+
+    url = ephy_bookmark_get_url (bookmark);
+    title = ephy_bookmark_get_title (bookmark);
+    if (strlen (title) == 0)
+      title = url;
+
+    tags = ephy_bookmark_get_tags (bookmark);
+    if (should_add_bookmark_to_model (self, data->query, title, url, tags)) {
+      EphySuggestion *suggestion;
+      g_autofree gchar *escaped_title = NULL;
+      g_autofree gchar *markup = NULL;
+
+      escaped_title = g_markup_escape_text (title, -1);
+      markup = dzl_fuzzy_highlight (escaped_title, data->query, FALSE);
+      suggestion = ephy_suggestion_new (markup, title, url);
+      ephy_suggestion_set_secondary_icon (suggestion, "starred-symbolic");
+
+      g_sequence_append (data->bookmarks, g_object_ref (suggestion));
+    }
+  }
+
+  query_collection_done (self, g_steal_pointer (&task));
+}
+
+static void
+history_query_completed_cb (EphyHistoryService *service,
+                            gboolean            success,
+                            gpointer            result_data,
+                            gpointer            user_data)
+{
+  GTask *task = user_data;
+  EphySuggestionModel *self;
+  QueryData *data;
+  GList *urls;
+
+  self = g_task_get_source_object (task);
+  data = g_task_get_task_data (task);
+  urls = (GList *)result_data;
+
+  if (strlen (data->query) > 0) {
+    for (const GList *p = urls; p != NULL; p = p->next) {
+      EphyHistoryURL *url = (EphyHistoryURL *)p->data;
+      EphySuggestion *suggestion;
+      g_autofree gchar *escaped_title = NULL;
+      g_autofree gchar *markup = NULL;
+      const gchar *title = url->title;
+
+      if (strlen (url->title) == 0)
+        title = url->url;
+
+      escaped_title = g_markup_escape_text (title, -1);
+
+      markup = dzl_fuzzy_highlight (escaped_title, data->query, FALSE);
+      suggestion = ephy_suggestion_new (markup, title, url->url);
+
+      g_sequence_append (data->history, g_steal_pointer (&suggestion));
+    }
+  }
+
+  query_collection_done (self, g_steal_pointer (&task));
+}
+
+#if SOUP_CHECK_VERSION (2, 99, 4)
+static void
+google_search_suggestions_cb (SoupSession  *session,
+                              GAsyncResult *result,
+                              gpointer      user_data)
+#else
+static void
+google_search_suggestions_cb (SoupSession *session,
+                              SoupMessage *msg,
+                              gpointer     user_data)
+#endif
+{
+  GTask *task = G_TASK (user_data);
+  EphySuggestionModel *self = g_task_get_source_object (task);
+  EphyEmbedShell *shell;
+  EphySearchEngineManager *manager;
+  QueryData *data;
+  JsonNode *node;
+  JsonArray *array;
+  JsonArray *suggestions;
+  EphySearchEngine *engine;
+  int added = 0;
+  g_autoptr (GBytes) body = NULL;
+#if SOUP_CHECK_VERSION (2, 99, 4)
+  SoupMessage *msg;
+
+  body = soup_session_send_and_read_finish (session, result, NULL);
+  if (!body)
+    goto out;
+
+  msg = soup_session_get_async_result_message (session, result);
+  if (soup_message_get_status (msg) != 200)
+    goto out;
+
+  /* FIXME: we don't have a way to get the status code */
+#else
+  if (msg->status_code != 200)
+    goto out;
+
+  body = g_bytes_new_static (msg->response_body->data, msg->response_body->length);
+#endif
+
+  shell = ephy_embed_shell_get_default ();
+  manager = ephy_embed_shell_get_search_engine_manager (shell);
+  engine = ephy_search_engine_manager_get_default_engine (manager);
+
+  node = json_from_string (g_bytes_get_data (body, NULL), NULL);
+  if (!node || !JSON_NODE_HOLDS_ARRAY (node)) {
+    g_warning ("Google search suggestion response is not a valid JSON object: %s", (const char *)g_bytes_get_data (body, NULL));
+    goto out;
+  }
+
+  array = json_node_get_array (node);
+  suggestions = json_array_get_array_element (array, 1);
+  data = g_task_get_task_data (task);
+
+  for (guint i = 0; i < json_array_get_length (suggestions) && added < 5; i++) {
+    EphySuggestion *suggestion;
+    const char *str = json_array_get_string_element (suggestions, i);
+    g_autofree char *address = NULL;
+    g_autofree char *escaped_title = NULL;
+    g_autofree char *markup = NULL;
+    const char *engine_name;
+
+    address = ephy_search_engine_build_search_address (engine, str);
+    escaped_title = g_markup_escape_text (str, -1);
+    markup = dzl_fuzzy_highlight (escaped_title, str, FALSE);
+    engine_name = ephy_search_engine_get_name (engine);
+    suggestion = ephy_suggestion_new (markup, engine_name, address);
+
+    g_sequence_append (data->google_suggestions, g_steal_pointer (&suggestion));
+    added++;
+  }
+
+out:
+  query_collection_done (self, g_steal_pointer (&task));
+}
+
+static void
+google_search_suggestions_query (EphySuggestionModel *self,
+                                 const gchar         *query,
+                                 GTask               *task)
+{
+  SoupMessage *msg;
+  g_auto (GStrv) split = g_strsplit (query, " ", -1);
+  g_autofree char *url = NULL;
+  g_autofree char *escaped_query = NULL;
+
+  if (g_strv_length (split) < 2) {
+    query_collection_done (self, g_steal_pointer (&task));
+    return;
+  }
+
+  escaped_query = g_markup_escape_text (query, -1);
+  url = g_strdup_printf ("http://suggestqueries.google.com/complete/search?client=firefox&q=%s", escaped_query);
+  msg = soup_message_new (SOUP_METHOD_GET, url);
+#if SOUP_CHECK_VERSION (2, 99, 4)
+  soup_session_send_and_read_async (self->session, msg, G_PRIORITY_DEFAULT, NULL,
+                                    (GAsyncReadyCallback)google_search_suggestions_cb,
+                                    g_steal_pointer (&task));
+  g_object_unref (msg);
+#else
+  soup_session_queue_message (self->session, g_steal_pointer (&msg), google_search_suggestions_cb, g_steal_pointer (&task));
+#endif
+}
+
 void
 ephy_suggestion_model_query_async (EphySuggestionModel *self,
                                    const gchar         *query,
@@ -582,8 +750,6 @@ ephy_suggestion_model_query_async (EphySuggestionModel *self,
                                    gpointer             user_data)
 {
   GTask *task = NULL;
-  char **strings;
-  GList *qlist = NULL;
   QueryData *data;
 
   g_assert (EPHY_IS_SUGGESTION_MODEL (self));
@@ -596,21 +762,38 @@ ephy_suggestion_model_query_async (EphySuggestionModel *self,
   data = query_data_new (query, include_search_engines);
   g_task_set_task_data (task, data, (GDestroyNotify)query_data_free);
 
-  /* Split the search string. */
-  strings = g_strsplit (query, " ", -1);
-  for (guint i = 0; strings[i]; i++)
-    qlist = g_list_append (qlist, g_strdup (strings[i]));
+  /* Google Search Suggestions */
+  if (data->scope == QUERY_SCOPE_ALL || data->scope == QUERY_SCOPE_SUGGESTIONS) {
+    if (g_settings_get_boolean (EPHY_SETTINGS_MAIN, EPHY_PREFS_USE_GOOGLE_SEARCH_SUGGESTIONS))
+      google_search_suggestions_query (self, query, task);
+    else
+      query_collection_done (self, task);
+  }
 
-  ephy_history_service_find_urls (self->history_service,
-                                  0, 0,
-                                  MAX_URL_ENTRIES, 0,
-                                  qlist,
-                                  EPHY_HISTORY_SORT_MOST_VISITED,
-                                  cancellable,
-                                  (EphyHistoryJobCallback)query_completed_cb,
-                                  task);
+  if (data->scope == QUERY_SCOPE_ALL || data->scope == QUERY_SCOPE_HISTORY) {
+    GList *qlist = NULL;
+    g_auto (GStrv) strings = NULL;
 
-  g_strfreev (strings);
+    strings = g_strsplit (data->query, " ", -1);
+
+    for (guint i = 0; strings[i]; i++)
+      qlist = g_list_append (qlist, g_strdup (strings[i]));
+
+    ephy_history_service_find_urls (self->history_service,
+                                    0, 0,
+                                    MAX_URL_ENTRIES, 0,
+                                    qlist,
+                                    EPHY_HISTORY_SORT_MOST_VISITED,
+                                    cancellable,
+                                    (EphyHistoryJobCallback)history_query_completed_cb,
+                                    task);
+  }
+
+  if (data->scope == QUERY_SCOPE_ALL || data->scope == QUERY_SCOPE_TABS)
+    tabs_query (self, data, task);
+
+  if (data->scope == QUERY_SCOPE_ALL || data->scope == QUERY_SCOPE_BOOKMARKS)
+    bookmarks_query (self, data, task);
 }
 
 gboolean
