@@ -20,19 +20,20 @@
 
 #pragma once
 
+#include "ephy-bookmark-states.h"
 #include "ephy-bookmarks-manager.h"
 #include "ephy-embed.h"
-#include "ephy-embed-event.h"
 #include "ephy-location-controller.h"
+#include "ephy-tab-view.h"
 #include "ephy-web-view.h"
 
-#include <gtk/gtk.h>
+#include <handy.h>
 
 G_BEGIN_DECLS
 
 #define EPHY_TYPE_WINDOW (ephy_window_get_type ())
 
-G_DECLARE_FINAL_TYPE (EphyWindow, ephy_window, EPHY, WINDOW, DzlApplicationWindow)
+G_DECLARE_FINAL_TYPE (EphyWindow, ephy_window, EPHY, WINDOW, HdyApplicationWindow)
 
 typedef enum
 {
@@ -46,8 +47,10 @@ typedef enum
 
 EphyWindow       *ephy_window_new                 (void);
 
-GtkWidget        *ephy_window_get_notebook        (EphyWindow *window);
-GtkWidget        *ephy_window_get_stack           (EphyWindow *window);
+EphyTabView      *ephy_window_get_tab_view        (EphyWindow *window);
+
+void              ephy_window_open_pages_view     (EphyWindow *window);
+void              ephy_window_close_pages_view    (EphyWindow *window);
 
 void              ephy_window_load_url            (EphyWindow *window,
                                                    const char *url);
@@ -56,6 +59,7 @@ void              ephy_window_set_zoom            (EphyWindow *window,
                                                    double zoom);
 
 void              ephy_window_activate_location   (EphyWindow *window);
+void              ephy_window_location_search     (EphyWindow *window);
 const char       *ephy_window_get_location        (EphyWindow *window);
 
 GtkWidget        *ephy_window_get_header_bar      (EphyWindow *window);
@@ -66,7 +70,7 @@ EphyWindowChrome  ephy_window_get_chrome          (EphyWindow *window);
 
 EphyLocationController  *ephy_window_get_location_controller (EphyWindow *window);
 
-EphyEmbedEvent   *ephy_window_get_context_event        (EphyWindow *window);
+WebKitHitTestResult *ephy_window_get_context_event     (EphyWindow *window);
 
 GtkWidget        *ephy_window_get_current_find_toolbar (EphyWindow *window);
 
@@ -81,9 +85,6 @@ void              ephy_window_set_default_position     (EphyWindow *window,
                                                         gint        y);
 void              ephy_window_show_fullscreen_header_bar (EphyWindow *window);
 
-int               ephy_window_get_position_for_new_embed (EphyWindow *window,
-                                                          EphyEmbed  *embed);
-
 void              ephy_window_update_entry_focus         (EphyWindow  *window,
                                                           EphyWebView *view);
 
@@ -94,4 +95,7 @@ gboolean          ephy_window_is_fullscreen              (EphyWindow *window);
 void              ephy_window_get_geometry               (EphyWindow   *window,
                                                           GdkRectangle *rectangle);
 
+void ephy_window_sync_bookmark_state (GtkWidget             *widget,
+                                      EphyBookmarkIconState  state);
+
 G_END_DECLS
