@@ -48,8 +48,8 @@ view_in_destination (EphyWindow      *window,
                      const char      *property_name,
                      LinkDestination  destination)
 {
-  EphyEmbedEvent *event;
-  GValue value = { 0, };
+  WebKitHitTestResult *hit_test_result;
+  g_autofree char *value = NULL;
   EphyEmbed *embed;
   EphyEmbed *new_embed;
   EphyWebView *new_view;
@@ -57,13 +57,13 @@ view_in_destination (EphyWindow      *window,
   EphyWindow *dest_window = window;
   EphyNewTabFlags flags = 0;
 
-  event = ephy_window_get_context_event (window);
-  g_assert (event != NULL);
+  hit_test_result = ephy_window_get_context_event (window);
+  g_assert (hit_test_result != NULL);
 
   embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
   g_assert (embed != NULL);
 
-  ephy_embed_event_get_property (event, property_name, &value);
+  g_object_get (hit_test_result, property_name, &value, NULL);
   switch (destination) {
     case NEW_WINDOW:
       dest_window = ephy_window_new ();
@@ -84,8 +84,7 @@ view_in_destination (EphyWindow      *window,
   session_state = webkit_web_view_get_session_state (WEBKIT_WEB_VIEW (ephy_embed_get_web_view (embed)));
   webkit_web_view_restore_session_state (WEBKIT_WEB_VIEW (new_view), session_state);
   webkit_web_view_session_state_unref (session_state);
-  ephy_web_view_load_url (new_view, g_value_get_string (&value));
-  g_value_unset (&value);
+  ephy_web_view_load_url (new_view, value);
 }
 
 void
@@ -117,25 +116,22 @@ popup_cmd_copy_link_address (GSimpleAction *action,
                              GVariant      *parameter,
                              gpointer       user_data)
 {
-  EphyEmbedEvent *event;
+  WebKitHitTestResult *hit_test_result;
   guint context;
   const char *address;
-  GValue value = { 0, };
 
-  event = ephy_window_get_context_event (EPHY_WINDOW (user_data));
-  g_assert (event != NULL);
+  hit_test_result = ephy_window_get_context_event (EPHY_WINDOW (user_data));
+  g_assert (hit_test_result != NULL);
 
-  context = ephy_embed_event_get_context (event);
+  context = webkit_hit_test_result_get_context (hit_test_result);
 
   if (context & WEBKIT_HIT_TEST_RESULT_CONTEXT_LINK) {
-    ephy_embed_event_get_property (event, "link-uri", &value);
-    address = g_value_get_string (&value);
+    address = webkit_hit_test_result_get_link_uri (hit_test_result);
 
     if (g_str_has_prefix (address, "mailto:"))
       address = address + 7;
 
     popup_cmd_copy_to_clipboard (EPHY_WINDOW (user_data), address);
-    g_value_unset (&value);
   }
 }
 
@@ -150,8 +146,44 @@ cancel_download_idle_cb (EphyDownload *download)
 typedef struct {
   char *title;
   EphyWindow *window;
+  EphyDownload *download;
 } SavePropertyURLData;
 
+static void
+filename_confirmed_cb (GtkFileChooser      *dialog,
+                       GtkResponseType      response,
+                       SavePropertyURLData *data)
+{
+  gtk_native_dialog_destroy (GTK_NATIVE_DIALOG (dialog));
+
+  if (response == GTK_RESPONSE_ACCEPT) {
+    char *uri;
+    WebKitDownload *webkit_download;
+
+    uri = gtk_file_chooser_get_uri (dialog);
+    ephy_download_set_destination_uri (data->download, uri);
+    g_free (uri);
+
+    webkit_download = ephy_download_get_webkit_download (data->download);
+    webkit_download_set_allow_overwrite (webkit_download, TRUE);
+
+    ephy_downloads_manager_add_download (ephy_embed_shell_get_downloads_manager (ephy_embed_shell_get_default ()),
+                                         data->download);
+    g_settings_set_string (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_LAST_DOWNLOAD_DIRECTORY,
+                           gtk_file_chooser_get_current_folder (dialog));
+  } else {
+    g_idle_add_full (G_PRIORITY_DEFAULT,
+                     (GSourceFunc)cancel_download_idle_cb,
+                     g_object_ref (data->download),
+                     g_object_unref);
+  }
+
+  g_free (data->title);
+  g_object_unref (data->window);
+  g_object_unref (data->download);
+  g_free (data);
+}
+
 static void
 filename_suggested_cb (EphyDownload        *download,
                        const char          *suggested_filename,
@@ -171,33 +203,9 @@ filename_suggested_cb (EphyDownload        *download,
   gtk_file_chooser_set_current_name (dialog, sanitized_filename);
   g_free (sanitized_filename);
 
-  if (gtk_native_dialog_run (GTK_NATIVE_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
-    char *uri;
-    WebKitDownload *webkit_download;
-
-    uri = gtk_file_chooser_get_uri (dialog);
-    ephy_download_set_destination_uri (download, uri);
-    g_free (uri);
-
-    webkit_download = ephy_download_get_webkit_download (download);
-    webkit_download_set_allow_overwrite (webkit_download, TRUE);
-
-    ephy_downloads_manager_add_download (ephy_embed_shell_get_downloads_manager (ephy_embed_shell_get_default ()),
-                                         download);
-    g_settings_set_string (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_LAST_DOWNLOAD_DIRECTORY, gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dialog)));
-  } else {
-    g_idle_add_full (G_PRIORITY_DEFAULT,
-                     (GSourceFunc)cancel_download_idle_cb,
-                     g_object_ref (download),
-                     g_object_unref);
-  }
-
-  g_object_unref (dialog);
-  g_free (data->title);
-  g_object_unref (data->window);
-  g_free (data);
-
-  g_object_unref (download);
+  g_signal_connect (dialog, "response",
+                    G_CALLBACK (filename_confirmed_cb), data);
+  gtk_native_dialog_show (GTK_NATIVE_DIALOG (dialog));
 }
 
 static void
@@ -205,26 +213,23 @@ save_property_url (const char *title,
                    EphyWindow *window,
                    const char *property)
 {
-  EphyEmbedEvent *event;
-  const char *location;
+  WebKitHitTestResult *hit_test_result;
+  g_autofree char *location = NULL;
   EphyDownload *download;
   SavePropertyURLData *data;
-  GValue value = G_VALUE_INIT;
 
-  event = ephy_window_get_context_event (window);
-  g_assert (event != NULL);
+  hit_test_result = ephy_window_get_context_event (window);
+  g_assert (hit_test_result != NULL);
 
-  ephy_embed_event_get_property (event, property, &value);
-  location = g_value_get_string (&value);
+  g_object_get (hit_test_result, property, &location, NULL);
   download = ephy_download_new_for_uri (location);
   data = g_new (SavePropertyURLData, 1);
   data->title = g_strdup (title);
   data->window = g_object_ref (window);
+  data->download = download;
   g_signal_connect (download, "filename-suggested",
                     G_CALLBACK (filename_suggested_cb),
                     data);
-
-  g_value_unset (&value);
 }
 
 void
@@ -268,20 +273,18 @@ popup_cmd_set_image_as_background (GSimpleAction *action,
                                    GVariant      *parameter,
                                    gpointer       user_data)
 {
-  EphyEmbedEvent *event;
+  WebKitHitTestResult *hit_test_result;
   const char *location;
   char *dest_uri, *dest, *base, *base_converted;
-  GValue value = { 0, };
   EphyDownload *download;
 
-  if (ephy_is_running_inside_flatpak ())
+  if (ephy_is_running_inside_sandbox ())
     return;
 
-  event = ephy_window_get_context_event (EPHY_WINDOW (user_data));
-  g_assert (event != NULL);
+  hit_test_result = ephy_window_get_context_event (EPHY_WINDOW (user_data));
+  g_assert (hit_test_result != NULL);
 
-  ephy_embed_event_get_property (event, "image-uri", &value);
-  location = g_value_get_string (&value);
+  location = webkit_hit_test_result_get_image_uri (hit_test_result);
 
   download = ephy_download_new_for_uri (location);
 
@@ -298,7 +301,6 @@ popup_cmd_set_image_as_background (GSimpleAction *action,
   g_signal_connect (download, "completed",
                     G_CALLBACK (background_download_completed), user_data);
 
-  g_value_unset (&value);
   g_free (base);
   g_free (base_converted);
   g_free (dest);
@@ -309,15 +311,12 @@ static void
 popup_cmd_copy_location (EphyWindow *window,
                          const char *property_name)
 {
-  EphyEmbedEvent *event;
-  const char *location;
-  GValue value = { 0, };
+  WebKitHitTestResult *hit_test_result;
+  g_autofree char *location = NULL;
 
-  event = ephy_window_get_context_event (window);
-  ephy_embed_event_get_property (event, property_name, &value);
-  location = g_value_get_string (&value);
+  hit_test_result = ephy_window_get_context_event (window);
+  g_object_get (hit_test_result, property_name, &location, NULL);
   popup_cmd_copy_to_clipboard (window, location);
-  g_value_unset (&value);
 }
 
 void
@@ -365,15 +364,15 @@ popup_cmd_link_in_incognito_window (GSimpleAction *action,
                                     GVariant      *parameter,
                                     gpointer       user_data)
 {
-  EphyEmbedEvent *event;
-  GValue value = { 0, };
+  WebKitHitTestResult *hit_test_result;
+  const char *link_uri;
 
-  event = ephy_window_get_context_event (EPHY_WINDOW (user_data));
-  g_assert (event != NULL);
+  hit_test_result = ephy_window_get_context_event (EPHY_WINDOW (user_data));
+  g_assert (hit_test_result != NULL);
 
-  ephy_embed_event_get_property (event, "link-uri", &value);
-  ephy_open_incognito_window (g_value_get_string (&value));
-  g_value_unset (&value);
+  link_uri = webkit_hit_test_result_get_link_uri (hit_test_result);
+
+  ephy_open_incognito_window (link_uri);
 }
 
 void
