@@ -23,6 +23,7 @@
 #include "ephy-search-provider.h"
 
 #include "ephy-bookmarks-manager.h"
+#include "ephy-embed-utils.h"
 #include "ephy-file-helpers.h"
 #include "ephy-prefs.h"
 #include "ephy-profile-utils.h"
@@ -168,6 +169,22 @@ handle_get_subsearch_result_set (EphyShellSearchProvider2  *skeleton,
   g_application_hold (G_APPLICATION (self));
   g_cancellable_reset (self->cancellable);
 
+  if (g_strv_length (terms) == 1) {
+    gboolean is_uri = ephy_embed_utils_address_is_valid (terms[0]);
+
+    if (is_uri) {
+      GPtrArray *results = g_ptr_array_new ();
+
+      g_ptr_array_add (results, g_strdup_printf ("special:load:%s", terms[0]));
+      g_ptr_array_add (results, NULL);
+      g_dbus_method_invocation_return_value (invocation,
+                                             g_variant_new ("(^as)", g_ptr_array_free (results, FALSE)));
+
+      g_application_release (G_APPLICATION (self));
+      return TRUE;
+    }
+  }
+
   gather_results_async (self, terms, self->cancellable,
                         complete_request, invocation);
 
@@ -199,6 +216,16 @@ handle_get_result_metas (EphyShellSearchProvider2  *skeleton,
       g_variant_builder_add (&builder, "{sv}",
                              "gicon", g_variant_new_string (APPLICATION_ID));
       g_variant_builder_close (&builder);
+    } else if (g_str_has_prefix (results[i], "special:load:")) {
+      g_variant_builder_open (&builder, G_VARIANT_TYPE ("a{sv}"));
+      g_variant_builder_add (&builder, "{sv}",
+                             "id", g_variant_new_string ("special:load"));
+      g_variant_builder_add (&builder, "{sv}",
+                             "name", g_variant_new_take_string (g_strdup_printf (_("Load “%s”"),
+                                                                                 results[i] + strlen ("special:load:"))));
+      g_variant_builder_add (&builder, "{sv}",
+                             "gicon", g_variant_new_string (APPLICATION_ID));
+      g_variant_builder_close (&builder);
     } else {
       EphySuggestion *suggestion;
       const char *title;
@@ -250,25 +277,10 @@ launch_search (EphySearchProvider  *self,
                guint                timestamp)
 {
   g_autofree char *search_string = NULL;
-  g_autofree char *query_param = NULL;
   g_autofree char *effective_url = NULL;
-  const char *address_search;
-  EphyEmbedShell *shell;
-  EphySearchEngineManager *search_engine_manager;
-
-  shell = ephy_embed_shell_get_default ();
-  search_engine_manager = ephy_embed_shell_get_search_engine_manager (shell);
-  address_search = ephy_search_engine_manager_get_default_search_address (search_engine_manager);
 
   search_string = g_strjoinv (" ", terms);
-  query_param = soup_form_encode ("q", search_string, NULL);
-
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wformat-nonliteral"
-  /* Format string under control of user input... but gsettings is trusted input. */
-  /* + 2 here is getting rid of 'q=' */
-  effective_url = g_strdup_printf (address_search, query_param + 2);
-#pragma GCC diagnostic pop
+  effective_url = ephy_embed_utils_autosearch_address (search_string);
 
   launch_uri (effective_url, timestamp);
 }
@@ -286,6 +298,8 @@ handle_activate_result (EphyShellSearchProvider2  *skeleton,
 
   if (strcmp (identifier, "special:search") == 0)
     launch_search (self, terms, timestamp);
+  else if (strcmp (identifier, "special:load") == 0)
+    launch_uri (terms[0], timestamp);
   else
     launch_uri (identifier, timestamp);
 
