@@ -41,6 +41,9 @@ void window_cmd_export_bookmarks                (GSimpleAction *action,
 void window_cmd_show_history                    (GSimpleAction *action,
                                                  GVariant      *parameter,
                                                  gpointer       user_data);
+void window_cmd_show_firefox_sync               (GSimpleAction *action,
+                                                 GVariant      *parameter,
+                                                 gpointer       user_data);
 void window_cmd_show_preferences                (GSimpleAction *action,
                                                  GVariant      *parameter,
                                                  gpointer       user_data);
@@ -131,6 +134,12 @@ void window_cmd_open_bookmark                   (GSimpleAction *action,
 void window_cmd_bookmark_page                   (GSimpleAction *action,
                                                  GVariant      *parameter,
                                                  gpointer       user_data);
+void window_cmd_bookmarks                       (GSimpleAction *action,
+                                                 GVariant      *parameter,
+                                                 gpointer       user_data);
+void window_cmd_show_downloads                  (GSimpleAction *action,
+                                                 GVariant      *parameter,
+                                                 gpointer       user_data);
 void window_cmd_zoom_in                         (GSimpleAction *action,
                                                  GVariant      *parameter,
                                                  gpointer       user_data);
@@ -176,24 +185,9 @@ void window_cmd_change_browse_with_caret_state  (GSimpleAction *action,
 void window_cmd_change_fullscreen_state         (GSimpleAction *action,
                                                  GVariant      *state,
                                                  gpointer       user_data);
-void window_cmd_tabs_previous                   (GSimpleAction *action,
-                                                 GVariant      *state,
-                                                 gpointer       user_data);
-void window_cmd_tabs_next                       (GSimpleAction *action,
-                                                 GVariant      *state,
-                                                 gpointer       user_data);
-void window_cmd_tabs_move_left                  (GSimpleAction *action,
-                                                 GVariant      *state,
-                                                 gpointer       user_data);
-void window_cmd_tabs_move_right                 (GSimpleAction *action,
-                                                 GVariant      *state,
-                                                 gpointer       user_data);
 void window_cmd_tabs_duplicate                  (GSimpleAction *action,
                                                  GVariant      *state,
                                                  gpointer       user_data);
-void window_cmd_tabs_detach                     (GSimpleAction *action,
-                                                 GVariant      *state,
-                                                 gpointer       user_data);
 void window_cmd_tabs_close                      (GSimpleAction *action,
                                                  GVariant      *parameter,
                                                  gpointer       user_data);
@@ -239,8 +233,14 @@ void window_cmd_tabs_pin                        (GSimpleAction *action,
 void window_cmd_tabs_unpin                      (GSimpleAction *action,
                                                  GVariant      *parameter,
                                                  gpointer       user_data);
+void window_cmd_change_tabs_mute_state          (GSimpleAction *action,
+                                                 GVariant      *state,
+                                                 gpointer       user_data);
 void window_cmd_import_passwords                (GSimpleAction *action,
                                                  GVariant      *parameter,
                                                  gpointer       user_data);
+void window_cmd_extensions                      (GSimpleAction *action,
+                                                 GVariant      *parameter,
+                                                 gpointer       user_data);
 
 G_END_DECLS
